package com.sos.scheduler.model.objects;

import java.io.File;
import org.apache.log4j.Logger;
import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.VirtualFileSystem.Interfaces.ISOSVirtualFile;
import com.sos.scheduler.model.SchedulerObjectFactory;
import com.sos.scheduler.model.tools.PathResolver;

/**
* \class JSObjInclude 
* 
* \brief JSObjInclude - 
* 
* \details
*
* \section JSObjInclude.java_intro_sec Introduction
*
* \section JSObjInclude.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author oh
* @version $Id$
* \see reference
*
* Created on 09.02.2011 14:36:27
 */

/**
 * @author oh
 *
 */
public class JSObjInclude extends Include {

	@SuppressWarnings("unused")
	private final String		conClassName	= "JSObjInclude";
	@SuppressWarnings("unused")
	private static final Logger	logger			= Logger.getLogger(JSObjInclude.class);
	
	public JSObjInclude (SchedulerObjectFactory schedulerObjectFactory) {
		super();
		objFactory = schedulerObjectFactory;
	}

	@Override
	public void setHotFolderSrc(ISOSVirtualFile pobjVirtualFile) {
		if (pobjVirtualFile != null) {
			if (getFile() != null) {	// the file attribute
				String fileName = replaceEnvVariables(getFile());
				if ( !PathResolver.isAbsolutePath(fileName) )
					throw new JobSchedulerException("only an absolute path for the attribute 'file' is allowed (path is '" + fileName + "')");
				ISOSVirtualFile vfInclude = objFactory.getFileHandleOrNull(fileName);
				super.setHotFolderSrc( vfInclude );
			} else {					// the live_file attribute
				File f = new File(pobjVirtualFile.getName());
				String baseFolder = f.getAbsolutePath().replace(f.getName(), "");
				String liveFolder = baseFolder;
				if (objFactory.getLiveFolderOrNull() != null) {
					liveFolder = objFactory.getLiveFolderOrNull().getHotFolderSrc().getName();
					String fileName = PathResolver.getAbsolutePath(liveFolder, baseFolder, getLiveFileResolved(getLiveFile()) );
					ISOSVirtualFile vfInclude = objFactory.getFileHandleOrNull(fileName);
					super.setHotFolderSrc( vfInclude );
				}
			}
		}
	}
	
	private String getLiveFileResolved(String liveFile) {
		String result = PathResolver.normalizePath(replaceEnvVariables(liveFile));
		if ( PathResolver.isAbsoluteWindowsPath(liveFile) )
			throw new JobSchedulerException("an absolute path for the attribute 'live_file' is not allowed (path is '" + result + "')");
		return result;
	}
	
	private String replaceEnvVariables(String rawName) {
		String result = rawName;
		for(String key : System.getenv().keySet()) {
			// logger.info(key + ": " + System.getenv(key));
			String searchFor = getRegExp("${" + key + "}");
			result = result.replaceAll(searchFor, getRegExp(System.getenv(key)) );
		}
		logger.info("after replacing: " + result);
		return result;
	}
	
	private String getRegExp(String rawString) {
		StringBuilder b = new StringBuilder();
		for(int i=0; i<rawString.length(); i++) {
			char ch = rawString.charAt(i);
			if ("\\.^$|?*+[]{}()".indexOf(ch) != -1)
				b.append("\\").append(ch);
			else
				b.append(ch);
		}
		return b.toString();
	}
	
	public String getContent() {
		return getHotFolderSrc().File2String();
	}

}
